TOOL_NAME = "Always A Free Instrument Slot"

------------------------------------------
--Set up Preferences file
------------------------------------------
--create xml
local options = renoise.Document.create {
  tool_active = false,
}
 --assign options-object to .preferences so renoise knows to load and update it with the tool
renoise.tool().preferences = options
------------------------------------------
--variable syntax 
--options.value_a.value


--Renoise Keybinds and menus
-------------------------------
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Ledger`s Scripts:"..TOOL_NAME,
  selected = function() return options.tool_active.value end,
  invoke = function()
             options.tool_active.value = not options.tool_active.value
             main()
           end  
}

--new document notifier means tool will not run until a renoise song exists (renoise app fully initialised)  
----------------------------------------------------------------------
renoise.tool().app_new_document_observable:add_notifier(function()main()end)

--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end

--function adds notifiers to observe changes in instrument name (loading an instrument onto a present empty slot)
--and the instrument[] list to watch for insert/delete actions
----------------------------
function add_all_notifiers()
----------------------------
  local song = renoise.song()
  local check = check_for_empty_slots_and_add_new
  --add notifiers for all instrument names
  ----------------------------------------
  for inst = 1,#song.instruments do
    local cur_inst = song:instrument(inst)
    --add for all instrument names
    if not cur_inst.name_observable:has_notifier(check) then
      cur_inst.name_observable:add_notifier(check)
    end
  end
  --add observable for instrument list events (insert remove etc.)
  if not song.instruments_observable:has_notifier(check) then
    song.instruments_observable:add_notifier(check)
  end
end

--remove notifiers
-------------------------------
function remove_all_notifiers()
-------------------------------
  --song
  local song = renoise.song()
  local check = check_for_empty_slots_and_add_new
  --remove notifiers
  for inst = 1,#song.instruments do
    local cur_inst = song:instrument(inst)
    --check and remove for all instrument names
    if cur_inst.name_observable:has_notifier(check) then
      cur_inst.name_observable:remove_notifier(check)
    end
  end
  --add observable for instrument list events (insert remove etc.)
  if song.instruments_observable:has_notifier(check) then
    song.instruments_observable:remove_notifier(check)
  end
end

--notifier function 
------------------------------------------------------
function check_for_empty_slots_and_add_new(event)
--------------------------------------------------------
  --rprint(event)
  
  --renoise song object
  local song = renoise.song()
  local num_insts = #song.instruments
  --return if we reached the max number of instruments (currently 255)
  if num_insts == renoise.Song.MAX_NUMBER_OF_INSTRUMENTS then
    return 
  end
  
  --variable for notifier fn
  local check = check_for_empty_slots_and_add_new
  
  --if there is only 1 instrument left, make sure it has a name notifier
  --which can occur if user deletes the last available instrument that renoise then re-adds
  if #song.instruments == 1 then
    local cur_inst = song:instrument(1)
    if not cur_inst.name_observable:has_notifier(check) then
      cur_inst.name_observable:add_notifier(check)
    end
  end
  
  --check all slot names for empties
  for i = 1, num_insts do
    if song:instrument(i).name == "" then
      --empty found so return
      return
    end
  end
  
  --insert new empty instrument if none have been found
  local new_inst = song:insert_instrument_at(num_insts + 1)
  --add notifier to the name of this instrument
  new_inst.name_observable:add_notifier(check)
  
  --increment num_insts as we now added one
  num_insts = num_insts + 1
  --if the event was deleting the last slot then
  --select the last slot that we added back in
  if (event ~= nil) and (event.index == num_insts) and
    event.type == "remove" then
    song.selected_instrument_index = num_insts
    -- status("Last Empty Slot Preserved")
  end
end

--called by 'new song' notifier and menu entry
---------------
function main()
---------------
  --renoise song object
  local song = renoise.song()
    
  if options.tool_active.value == true then
    --add notifiers
    add_all_notifiers()
    --do a first run of the function to add any new slots if needed
    check_for_empty_slots_and_add_new()
    --status
    status("Enabled")
    return 
  end  
  --if we got here, options.tool_active.value will be false
  --so remove notifiers
  remove_all_notifiers()
  --status
  status("Disabled")
end


 
